@@ -18,6 +18,9 @@
  * This file contains C implementations of some of the functions needed by the
  * bson module. If possible, these implementations should be used to speed up
  * BSON encoding and decoding.
+ *
+ * TODO The filename is a bit of a misnomer now - probably should be something
+ * like _cspeedupsmodule - we do more than just BSON stuff in this C module.
  */
 
 
@@ -942,6 +945,94 @@ static PyObject* _cbson_dict_to_bson(PyObject* self, PyObject* args) {
     return result;
 }
 
+/* add a lastError message on the end of the buffer.
+ * returns 0 on failure */
+static int add_last_error(bson_buffer* buffer, int request_id) {
+                                 /* message length: 62 */
+    if (!buffer_write_bytes(buffer, "\x3E\x00\x00\x00", 4) ||
+        !buffer_write_bytes(buffer, (const char*)&request_id, 4) ||
+        !buffer_write_bytes(buffer,
+                            "\x00\x00\x00\x00" /* responseTo */
+                            "\xd4\x07\x00\x00" /* opcode */
+                            "\x00\x00\x00\x00" /* options */
+                            "admin.$cmd\x00"   /* collection name */
+                            "\x00\x00\x00\x00" /* skip */
+                            "\xFF\xFF\xFF\xFF" /* limit (-1) */
+                            "\x17\x00\x00\x00" /* {getlasterror: 1} */
+                            "\x10getlasterror" /* ... */
+                            "\x00\x01\x00\x00" /* ... */
+                            "\x00\x00",        /* ... */
+                            54)) {
+        return 0;
+    }
+    return 1;
+}
+
+static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
+    /* NOTE just using a random number as the request_id */
+    int request_id = rand();
+    char* collection_name;
+    int collection_name_length;
+    PyObject* docs;
+    int i;
+    unsigned char check_keys;
+    unsigned char safe;
+    bson_buffer* buffer;
+    int length_location;
+    PyObject* result;
+
+    if (!PyArg_ParseTuple(args, "s#Obb",
+                          &collection_name,
+                          &collection_name_length,
+                          &docs, &check_keys, &safe)) {
+        return NULL;
+    }
+
+    buffer = buffer_new();
+    if (!buffer) {
+        return NULL;
+    }
+
+    // save space for message length
+    length_location = buffer_save_bytes(buffer, 4);
+    if (length_location == -1 ||
+        !buffer_write_bytes(buffer, (const char*)&request_id, 4) ||
+        !buffer_write_bytes(buffer,
+                            "\x00\x00\x00\x00"
+                            "\xd2\x07\x00\x00"
+                            "\x00\x00\x00\x00",
+                            12) ||
+        !buffer_write_bytes(buffer,
+                            collection_name,
+                            collection_name_length + 1)) {
+        buffer_free(buffer);
+        return NULL;
+    }
+
+    for (i = 0; i < PyList_Size(docs); i++) {
+        PyObject* doc = PyList_GetItem(docs, i);
+        if (!write_dict(buffer, doc, check_keys)) {
+            buffer_free(buffer);
+            return NULL;
+        }
+    }
+
+    memcpy(buffer->buffer + length_location, &buffer->position, 4);
+
+    if (safe) {
+        if (!add_last_error(buffer, request_id)) {
+            buffer_free(buffer);
+            return NULL;
+        }
+    }
+
+    /* objectify buffer */
+    result = Py_BuildValue("is#", request_id,
+                           buffer->buffer, buffer->position);
+    buffer_free(buffer);
+    return result;
+}
+
 static PyObject* get_value(const char* buffer, int* position, int type) {
     PyObject* value;
     switch (type) {
@@ -1357,6 +1448,8 @@ static PyMethodDef _CBSONMethods[] = {
      "convert a BSON string to a SON object."},
     {"_to_dicts", _cbson_to_dicts, METH_O,
      "convert binary data to a sequence of SON objects."},
+    {"_insert_message", _cbson_insert_message, METH_VARARGS,
+     "create an insert message to be sent to MongoDB"},
     {NULL, NULL, 0, NULL}
 };
 
