@@ -19,7 +19,9 @@ import os
 import sys
 sys.path[0:0] = [""]
 
-from pymongo.errors import ConnectionFailure, InvalidName
+from nose.plugins.skip import SkipTest
+
+from pymongo.errors import ConnectionFailure, InvalidName, AutoReconnect
 from pymongo.database import Database
 from pymongo.connection import Connection
 
@@ -128,6 +130,29 @@ class TestConnection(unittest.TestCase):
 
         self.assertRaises(TypeError, iterate)
 
+    # TODO this test is probably very dependent on the machine its running on
+    # due to timing issues, but I want to get something in here.
+    def test_low_network_timeout(self):
+        c = None
+        i = 0
+        while c is None and i < 1000:
+            try:
+                c = Connection(self.host, self.port, network_timeout=0.0001)
+            except AutoReconnect:
+                i += 1
+        if i == 1000:
+            raise SkipTest()
+
+        coll = c.pymongo_test.test
+
+        for _ in range(1000):
+            try:
+                coll.find_one()
+            except AutoReconnect:
+                pass
+            except AssertionError:
+                self.fail()
+
     def test_socket_timeout(self):
         no_timeout = Connection(self.host, self.port)
         timeout = Connection(self.host, self.port, network_timeout=0.1)
