@@ -21,6 +21,8 @@ import sys
 import itertools
 sys.path[0:0] = [""]
 
+from nose.plugins.skip import SkipTest
+
 from pymongo.errors import InvalidOperation, OperationFailure
 from pymongo.cursor import Cursor
 from pymongo.database import Database
@@ -444,7 +446,7 @@ class TestCursor(unittest.TestCase):
         self.db.test.remove({})
         self.db.test.save({"x": 1})
 
-        if not version.at_least(self.db.connection(), (1, 1)):
+        if not version.at_least(self.db.connection(), (1, 1, 3, -1)):
             for _ in self.db.test.find({}, ["a"]):
                 self.fail()
 
@@ -525,10 +527,13 @@ class TestCursor(unittest.TestCase):
         self.assertRaises(IndexError, lambda x: self.db.test.find()[x], -1)
         self.assertRaises(IndexError, lambda x: self.db.test.find()[x], 100)
 
-    def test_len(self):
+    def test_count_with_limit_and_skip(self):
+        if not version.at_least(self.db.connection(), (1, 1, 4, -1)):
+            raise SkipTest()
+
         def check_len(cursor, length):
-            self.assertEqual(len(list(cursor)), len(cursor))
-            self.assertEqual(length, len(cursor))
+            self.assertEqual(len(list(cursor)), cursor.count(True))
+            self.assertEqual(length, cursor.count(True))
 
         self.db.drop_collection("test")
         for i in range(100):
@@ -547,6 +552,9 @@ class TestCursor(unittest.TestCase):
         check_len(self.db.test.find().limit(10).skip(95), 5)
         check_len(self.db.test.find()[95:105], 5)
 
+    def test_len(self):
+        self.assertRaises(TypeError, len, self.db.test.find())
+
     def test_properties(self):
         self.assertEqual(self.db.test, self.db.test.find().collection)
 
